from datetime import date

from pyluach import dates  # type: ignore

from .base import HolidayGenerator
from .base import HolidayRelativeGenerator as HRG


class StableHebrewDateGenerator(HolidayGenerator):
    def __init__(self, month, day):
        self.month = month
        self.day = day

    def date_gen(self, after: date):
        cur_date = dates.GregorianDate(after.year, after.month, after.day).to_heb()

        this_year = dates.HebrewDate(cur_date.year, self.month, self.day)

        if cur_date > this_year:
            cur_date = dates.HebrewDate(cur_date.year + 1, self.month, self.day)
        else:
            cur_date = this_year

        while True:
            yield cur_date.to_pydate()
            cur_date = dates.HebrewDate(cur_date.year + 1, self.month, self.day)


RoshHashannah = HRG(StableHebrewDateGenerator(7, 1))
YomKippur = HRG(StableHebrewDateGenerator(7, 10))
Passover = HRG(StableHebrewDateGenerator(1, 15))
FirstNightHanukkah = HRG(StableHebrewDateGenerator(9, 25))

__all__ = ["RoshHashannah", "YomKippur", "Passover", "FirstNightHanukkah"]
