from datetime import date

from hijri_converter import convert

from .base import HolidayGenerator


class StableIslamicDateGenerator(HolidayGenerator):
    def __init__(self, month: int, day: int):
        self.month = month
        self.day = day

    def date_gen(self, after: date):
        cur_date = convert.Gregorian.fromdate(after).to_hijri()

        this_year = convert.Hijri(cur_date.year, self.month, self.day)

        if cur_date > this_year:
            cur_date = convert.Hijri(cur_date.year + 1, self.month, self.day)
        else:
            cur_date = this_year

        while True:
            yield date(*cur_date.to_gregorian().datetuple())
            cur_date = convert.Hijri(cur_date.year + 1, self.month, self.day)


Ashura = StableIslamicDateGenerator(1, 10)
Ramadan = StableIslamicDateGenerator(9, 1)
EidAlFatr = StableIslamicDateGenerator(10, 1)
EidAlAdha = StableIslamicDateGenerator(12, 10)

__all__ = ["Ashura", "Ramadan", "EidAlFatr", "EidAlAdha"]
