from datetime import date, timedelta

from .base import (
    FloatingDateGenerator,
    HolidayRelativeGenerator,
    NearestWeekdayGenerator,
    StableDateGenerator,
)


class MemorialDayGenerator(FloatingDateGenerator):
    def date_for_year(self, year: int):
        latest_date = date(year, 5, 31)
        daysdiff = latest_date.weekday()
        return latest_date - timedelta(days=daysdiff)


class ThanksgivingGenerator(FloatingDateGenerator):
    def date_for_year(self, year: int):
        earliest_date = date(year, 11, 22)
        daysdiff = (7 - earliest_date.weekday() + 3) % 7
        return earliest_date + timedelta(days=daysdiff)


class MartinLutherKingJrGenerator(FloatingDateGenerator):
    def date_for_year(self, year: int):
        earliest_date = date(year, 1, 15)
        daysdiff = (7 - earliest_date.weekday()) % 7
        return earliest_date + timedelta(days=daysdiff)


class PresidentsDayGenerator(FloatingDateGenerator):
    def date_for_year(self, year: int):
        earliest_date = date(year, 2, 15)
        daysdiff = (7 - earliest_date.weekday()) % 7
        return earliest_date + timedelta(days=daysdiff)


class LaborDayGenerator(FloatingDateGenerator):
    def date_for_year(self, year: int):
        earliest_date = date(year, 9, 1)
        daysdiff = (7 - earliest_date.weekday()) % 7
        return earliest_date + timedelta(days=daysdiff)


class IndigenousPeoplesDayGenerator(FloatingDateGenerator):
    def date_for_year(self, year: int):
        earliest_date = date(year, 10, 8)
        daysdiff = (7 - earliest_date.weekday()) % 7
        return earliest_date + timedelta(days=daysdiff)


class ElectionDayGenerator(FloatingDateGenerator):
    def date_for_year(self, year: int):
        earliest_date = date(year, 11, 2)
        daysdiff = (7 - earliest_date.weekday() + 1) % 7
        return earliest_date + timedelta(days=daysdiff)


NewYears = StableDateGenerator(1, 1)
NewYearsEve = StableDateGenerator(12, 31)
MartinLutherKingJrDay = MartinLutherKingJrGenerator()
PresidentsDay = PresidentsDayGenerator()
ValentinesDay = StableDateGenerator(2, 14)
StPatricksDay = StableDateGenerator(3, 17)
AprilFoolsDay = StableDateGenerator(4, 1)
MemorialDay = MemorialDayGenerator()
IndependenceDay = StableDateGenerator(7, 4)
IndependenceDayObserved = NearestWeekdayGenerator(IndependenceDay)
LaborDay = LaborDayGenerator()
IndigenousPeoplesDay = IndigenousPeoplesDayGenerator()
Halloween = StableDateGenerator(10, 31)
VeteransDay = StableDateGenerator(11, 11)
VeteransDayObserved = NearestWeekdayGenerator(VeteransDay)
Thanksgiving = ThanksgivingGenerator()
BlackFriday = HolidayRelativeGenerator(Thanksgiving, timedelta(days=1))
ElectionDay = ElectionDayGenerator()

__all__ = [
    "NewYears",
    "NewYearsEve",
    "MartinLutherKingJrDay",
    "ValentinesDay",
    "StPatricksDay",
    "AprilFoolsDay",
    "MemorialDay",
    "IndependenceDay",
    "IndependenceDayObserved",
    "LaborDay",
    "IndigenousPeoplesDay",
    "Halloween",
    "VeteransDay",
    "VeteransDayObserved",
    "Thanksgiving",
    "BlackFriday",
    "ElectionDay",
    "PresidentsDay",
]
