from datetime import date, timedelta
from itertools import dropwhile
from typing import Container

from .base import HolidayGenerator

ONE_DAY = timedelta(days=1)


def next_non_holiday(
    base_date: date, holiday_gen: HolidayGenerator, days_of_week: Container[int] = None
) -> date:
    """\
    Given a `datetime.date`, a `HolidayGenerator`, and an acceptable set of
    days of the week on which to scan, return the next date that is on an
    acceptable day of the week that is not a recognized holiday.
    """
    if days_of_week is None:
        days_of_week = frozenset(range(0, 7))

    holidays = holiday_gen.dates(after=base_date)
    next_holiday = next(holidays)

    while True:
        if next_holiday < base_date:
            next_holiday = next(dropwhile(lambda d: d < base_date, holidays))

        if (
            base_date.weekday() not in days_of_week or base_date == next_holiday
        ):  # noqa: E501
            base_date += ONE_DAY
        else:
            return base_date
